"""
Usage:
  lumicalc3 lumi (overview | byls | bylsXing) [options] 

Options:
  -h, --help      Show this screen
  -c CONNECT       Connect string to lumiDB 
                   [default: frontier://LumiCalc/CMS_LUMI_PROD]
  -p AUTHPATH      Path to authentication.xml
  -n SCALEFACTOR   Scale factor to results [default: 1.0]
  -f FILLNUM       Fill number
  -r RUNNUMBER     Run number
  -i INPUTFILE     Input selection file
  -o OUTPUTFILE    Output file
  --siteconfpath=SITECONFPATH 
                   Path to SITECONF/local/JobConfig/site-local-config.xml
                   [default: $CMS_PATH] 
  --amodetag=AMODETAG 
                   Accelerator mod choices 
  --beamstatus=BEAMSTATUS
                   Beam mode choices
  --beamenergy=BEAMENERGY
                   Single beam energy in GeV
  --beamfluctuation=BEAMFLUCTUATION [default: .2]
                   Fluctuation in fraction allowed to beam energy [default: 0.2]
  --minBiasXsec=MINBIASXSEC
                   Minbias cross-section in ub [default: 69300.0]
  --datatag=DATATAG
                   Version of data 
  --begin=BEGIN    Min start time 
  --end=END        Max start time
  --headerfile=HEADERFILE
                   Write command header output to specified file
  --hltpath=HLTPATH
                   Hlt path name/pattern for effective luminosity
  --algo=LUMIALGO  Lumi algorithm priority list 
                   [default: PLX|HFOC1] 
  --normtag=NORMTAG
                   Version of lumi correction/calibration
  --without-correction
                   Without correction/calibration  
                   [default: False] 
  --nowarning      Supress warnings
                   [default: False]
  --debug          Debug

"""
import os
from docopt import docopt
from schema import Schema, And, Or, Use

def validate(optdict,amodetagChoices,algoChoices):
    result={}
    schema = Schema({
     '--algo': And(str,lambda s: set(map(str.upper,s.split('|'))).issubset(set(algoChoices)), error='--algo choice unit must be in '+str(algoChoices) ) ,       
     '--amodetag':  Or(None,And(str,lambda s: s.upper() in amodetagChoices), error='--amodetag choice must be in '+str(amodetagChoices) ),
     '--beamenergy': Or(None,And(Use(int), lambda n: n>0), error='--beamenergy should be integer >0'),
     '--beamfluctuation': And(Use(float), lambda f: f>0 and f<1.0, error='--beamfluctuation should be float >0'),
     '--beamstatus': Or(None, And(Use(str), lambda s: s.upper()), error='--beamstatus should be string'),
     '--begin': Or(None, Use(str), error='--begin should be string'),
     '--end': Or(None, Use(str), error='--end should be string'),
     '--minBiasXsec': And(Use(float), lambda f: f>=0., error='--minBiasXsec should be float >=0'),
     '--siteconfpath': Or(None, str, error='--siteconfpath should be string'),
     '-c': str,
     '-f': Or(None, And(Use(int), lambda n: n>1000), error='-f FILLNUMBER should be integer >1000'), 
     '-n': And(Use(float), lambda f: f>0, error='-n SCALEFACTOR should be float >0'),      
     '-r': Or(None, And(Use(int), lambda n: n>100000), error='-r RUNNUMBER should be integer >100000'),
     str:object # catch all
    })
    result=schema.validate(optdict)
    return result

if __name__ == '__main__':
    args = docopt(__doc__,options_first=True)
    print args




import ast
import numpy as np


class FunctionFactory(object):

    
    def hcaldrift(self,ivalue,rawavglumi,intglumi,nbx,a0=1.0,a1=0.0,a2=0.0,drift=0.0,c1=0.0,afterglowthresholds=''):
        '''
        pain . later maybe
        
        '''
        avglumi = 0.
        if c1 and nbx>0:
            avglumi = c1*rawavglumi/nbx
        driftterm = 1.0
        if drift and intglumi: driftterm=1.0+drift*intglumi
        resultfactor = a0 * driftterm * self.inversepoly1d(avglumi,np.array([a2,a1,1])) 
        result = ivalue*resultfactor
        if afterglowthresholds:
            result = self.afterglow(result,nbx,afterglowthresholds)
        return result
            
    def poly1d(self,*args,**kwds):
        ivalue = args[0]
        coefs = kwds['coefs']
        if isinstance(coefs,str):
            coefs = np.fromstring(coefs, dtype=np.float, sep=',')
        f = np.poly1d(coefs)
        return f(ivalue)
    
    def poly1d_tostring(self,*args,**kwds):
        coefs = kwds['coefs']
        if isinstance(coefs,str):
            coefs = np.fromstring(coefs, dtype=np.float, sep=',')
        f = np.poly1d(coefs)
        return 'poly1d: %s'%f
    
    def inversepoly1d(self,*args,**kwds):
        return 1./self.poly1d(*args,**kwds)

    def inversepoly1d_tostring(self,*args,**kwds):
        return 'inversepoly1d: 1/%s'%(self.poly1d_tostring(*args,**kwds))
    
    def afterglow(self,*args,**kwds):
        ivalue = args[0]
        ncollidingbx = args[1]
        afterglowthresholds = kwds['afterglowthresholds']
        afterglow = 1.
        if isinstance(afterglowthresholds,str):
            if afterglowthresholds[-1]!=',': afterglowthresholds+=','
            afterglowthresholds = np.array(ast.literal_eval(afterglowthresholds))
        for (bxthreshold,c) in afterglowthresholds:
            if ncollidingbx >= bxthreshold:
                afterglow = c
        return ivalue*afterglow
    
    def afterglow_tostring(self,*args,**kwds):
        return 'afterglow: %s'%(kwds['afterglowthresholds'])

    def poly1dWafterglow(self,*args,**kwds):
        '''
        poly1d*afterglow
        '''
        newargs = [ self.poly1d(*args,**kwds) ]
        if len(args)>1: newargs = newargs+list(args[1:])
        result = self.afterglow(*tuple(newargs),**kwds)
        return result

    def poly1dWafterglow_tostring(self,*args,**kwds):
        return '('+poly1d_tostring(**kwds)+')*('+afterglow_tostring(**kwds)+')'

    def inversepoly1dWafterglow(self,*args,**kwds):
        '''
        poly1d*afterglow
        '''
        result = self.inversepoly1d(*args,**kwds)
        args[0] = result
        result = self.afterglow(*args,**kwds)
        return result

    def inversepoly1dWafterglow_tostring(self,*args,**kwds):
        return '('+inversepoly1d_tostring(**kwds)+')*('+afterglow_tostring(**kwds)+')'
    
    def chooser(self,*args,**kwds):        
        return args[0]

    def chooser_tostring(self,*args,**kwds):
        return 'chooser: %s'%(args[0])
        
def FunctionCaller(funcName,*args,**kwds):
    fac = FunctionFactory()
    try:
        myfunc = getattr(fac,funcName,None)
    except AttributeError:
        print '[ERROR] unknown correction function '+funcName
        raise
    if callable(myfunc):
        return myfunc(*args,**kwds)
    else:
        raise ValueError('uncallable function '+funcName)

    
if __name__=='__main__':
    ivalue = np.array([1.,2.,35.])
    result = FunctionCaller('poly1d',ivalue,coefs=np.array([2.,0.]))
    print result
    print FunctionCaller('poly1d_tostring',coefs=np.array([2.,0.]))    
    result = FunctionCaller('inversepoly1d',ivalue,coefs=np.array([2.,0.]))
    print result
    

    result = FunctionCaller('chooser','aa')
    print result

    result = FunctionCaller('afterglow',ivalue,3,afterglowthresholds=[(1,2),(2,5)])
    print result
 
    result = FunctionCaller('afterglow',ivalue,3,afterglowthresholds='(1,2),')
    print result

    print FunctionCaller('afterglow_tostring',afterglowthresholds='(1,2),(2,5)')



    ##accumulative function sequence
    fs = [ ['poly1d',[ivalue],{'coefs':np.array([2.,0.])}], ['afterglow',[ivalue,3], {'afterglowthresholds':'(1,2),(2,5)'}] ]

    result = None
    for f in fs:
        f_name = f[0]        
        f_args = f[1]
        if result is not None:
            f_args[0] = result
        f_kwds = f[2]
        result = FunctionCaller(f_name,*f_args,**f_kwds)
        print 'applying function: %s'%(FunctionCaller(f_name+'_tostring',**f_kwds))
    print 'final result ',result
    
